package handlers

import (
	"gralias/broker"
	"gralias/config"
	"gralias/models"
	"gralias/repos"
	"html/template"
	"log/slog"
	"net/http"
	"os"
)

var (
	log *slog.Logger
	cfg *config.Config
	// memcache cache.Cache
	Notifier *broker.Broker
	repo     repos.AllRepos
)

func init() {
	log = slog.New(slog.NewJSONHandler(os.Stderr, &slog.HandlerOptions{
		Level:     slog.LevelDebug,
		AddSource: true,
	}))
	cfg = config.LoadConfigOrDefault("")
	Notifier = broker.Notifier
	// repo = repos.NewRepoProvider("sqlite3://../gralias.db")
	repo = repos.RP
	recoverBots()
}

func HandlePing(w http.ResponseWriter, r *http.Request) {
	if _, err := w.Write([]byte("pong")); err != nil {
		log.Error("failed to write ping response", "error", err)
	}
}

func HandleHome(w http.ResponseWriter, r *http.Request) {
	tmpl, err := template.ParseGlob("components/*.html")
	if err != nil {
		abortWithError(w, err.Error())
		return
	}
	fi, err := getFullInfoByCtx(r.Context())
	if err != nil {
		log.Error("failed to fetch fi", "error", err)
	}
	// there must be a better way
	if fi != nil && fi.Room != nil && fi.Room.ID != "" && fi.State != nil {
		fi.Room.UpdateCounter()
		if fi.State.Role == "mime" {
			fi.Room.MimeView() // there must be a better way
		} else {
			fi.Room.GuesserView()
		}
	}
	if fi != nil && fi.Room == nil {
		rooms, err := repo.RoomList(r.Context())
		if err != nil {
			log.Error("failed to list rooms;", "error", err)
		}
		fi.List = rooms
	}
	if err := tmpl.ExecuteTemplate(w, "base", fi); err != nil {
		log.Error("failed to exec templ;", "error", err, "templ", "base")
	}
}

func HandleExit(w http.ResponseWriter, r *http.Request) {
	tmpl, err := template.ParseGlob("components/*.html")
	if err != nil {
		abortWithError(w, err.Error())
		return
	}
	fi, err := getFullInfoByCtx(r.Context())
	if err != nil {
		abortWithError(w, err.Error())
		return
	}
	if fi.Room.IsRunning {
		abortWithError(w, "cannot leave when game is running")
		return
	}
	var creatorLeft bool
	if fi.Room.CreatorName == fi.State.Username {
		creatorLeft = true
	}
	exitedRoom := fi.ExitRoom()
	if creatorLeft {
		if err := repo.RoomDeleteByID(r.Context(), exitedRoom.ID); err != nil {
			log.Error("failed to remove room", "error", err)
		}
		notify(models.NotifyRoomListUpdate, "")
	}
	if err := repo.PlayerExitRoom(r.Context(), fi.State.Username); err != nil {
		log.Error("failed to exit room", "error", err)
		abortWithError(w, err.Error())
		return
	}
	if err := repo.RoomUpdate(r.Context(), exitedRoom); err != nil {
		log.Error("failed to update room", "error", err)
		abortWithError(w, err.Error())
		return
	}
	fi.List, err = repo.RoomList(r.Context())
	if err != nil {
		abortWithError(w, err.Error())
		return
	}
	if err := tmpl.ExecuteTemplate(w, "base", fi); err != nil {
		log.Error("failed to exec templ;", "error", err, "templ", "base")
	}
}
